package com.audinate.controller;

import com.audinate.AudinateUtils;
import com.audinate.DanteDapi;
import com.audinate.Pair;
import com.audinate.Util;
import com.audinate.app.IJavaEnvironment;
import com.audinate.app.JavaEnvironment;
import com.audinate.cmm.CmmConfig;
import com.audinate.cmm.CmmInterface;
import com.audinate.cmm.CmmState;
import com.audinate.cmm.ICmmClient;
import com.audinate.cmm.ICmmClientListener;
import com.audinate.cmm.ICmmClients;
import com.audinate.controller.logic.ControllerLogic;
import com.audinate.dante.DeviceUtils;
import com.audinate.dante.IDevice;
import com.audinate.dante.logic.ILogicDevicesController;
import com.audinate.exception.AudinateException;
import com.audinate.gui.controller.ControllerFrame;
import com.audinate.gui.controller.DeviceFrame;
import com.audinate.gui.controller.DeviceFrames;
import com.audinate.gui.controller.MenuFrame;
import com.audinate.gui.controller.MillauLockDialog;
import com.audinate.gui.controller.StaleAudioConnectionsDialog;
import com.audinate.gui.dante.ISwingDevicesController;
import com.audinate.gui.i18n.ILocalisable;
import com.audinate.gui.themes.ThemeSelector;
import com.audinate.gui.themes.ThemedImage;
import com.audinate.gui.util.ImageLibrary;
import com.audinate.gui.util.ScreenUtils;
import com.audinate.model.json.JsonNull;
import com.audinate.model.json.JsonObject;
import com.audinate.model.json.JsonUtil;
import com.audinate.net.NetworkInterface;
import com.audinate.presets.legacy.LegacyUtils;
import com.audinate.timer.ISchedulableTimer;
import com.audinate.timer.ISchedulableTimerHandler;
import com.audinate.urlscheme.UrlSchemeHandler;
import com.audinate.util.IObservable;
import com.audinate.util.IObserver;
import com.audinate.util.IdleStateListener;
import com.audinate.util.IdleStateManager;
import com.audinate.util.Log;
import com.audinate.util.ResourceBundleUtils;
import com.audinate.util.SafeEquals;
import com.audinate.util.Version;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.Container;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Desktop.Action;
import java.awt.desktop.OpenURIEvent;
import java.awt.desktop.OpenURIHandler;
import java.awt.event.ComponentListener;
import java.awt.event.WindowListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.lang.ProcessBuilder.Redirect;
import java.net.URI;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.file.Paths;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Queue;
import java.util.ResourceBundle;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

public class DanteController implements IControllerConstants, ILogicDevicesController, ISwingDevicesController, ICmmClientListener, ISchedulableTimerHandler, ILocalisable {
   private static final String STORE_KEY_URL = "url";
   private static final String HELP_ICON_KEY = "OptionPane.questionIcon";
   private static final String DIALOG_WARN_VALUE = "warn";
   private static final String DIALOG_IGNORE_VALUE = "ignore";
   public static final boolean DANTE_CONTROLLER_SHARED_LIBRARY = true;
   public static final boolean DEBUG_UI_DEFAULTS = false;
   public static final boolean DEBUG_RUN = false;
   public static final boolean DEBUG_EXIT = false;
   public static final int DANTE_AGENT_RESTARTS = 5;
   private static final String DANTE_CONTROLLER_AGENT = "DanteControllerAgent";
   private static final String DANTE_ARGO_GATEWAY = "DanteGateway";
   private static final String PROP_AGENT_SEARCH_PATH = "dc.agent.path";
   private static final String PROP_TRANSLATOR_SEARCH_PATH = "dc.translator.path";
   private static final String PROP_PLUGIN_SEARCH_PATH = "dc.plugin.path";
   private static final String PROP_DYLIB_SEARCH_PATH = "dc.dylib.path";
   private static final int STATE_STOPPED = 0;
   private static final int STATE_STOPPING = 1;
   private static final int STATE_STARTED = 2;
   private static final int STATE_CMM_INIT = 3;
   private static final int STATE_STARTING = 4;
   private static final Integer SOURCE_URL_HANDLER = 0;
   private static final Integer SOURCE_PERSISTED = 1;
   private long mStartTime = System.currentTimeMillis();
   private ControllerConfig mConfig = new ControllerConfig();
   private ControllerApplication mApplication;
   private ControllerLogic mLogic;
   private LogicModel mLogicModel;
   private ControllerFrame mControllerFrame;
   private DeviceFrames mDeviceFrames;
   private String mAgentPath;
   private String mTranslatorPath;
   private Process mAgentProcess;
   private Process mTranslatorProcess;
   private List<String> mPluginSearchPaths;
   private int mAgentRestartCount = 0;
   private ICmmClient mCmmClient;
   private ICmmClients mCmmClients;
   private int mAgentWebserverPort;
   private int mAgentWebsocketPort;
   private int mAgentRemoteComponentPort = 0;
   private NetworkInterface mArgoInterface;
   private ResourceBundle mBundle;
   private IObserver mCmmClientsObserver = new IObserver() {
      public void update(IObservable var1, Object var2) {
         if (var1 instanceof ICmmClients) {
            DanteController.this.setCmmClient(DanteController.this.mCmmClients.getSelectedClient());
         }

      }
   };
   private IObserver mInterfaceManagerObserver = new IObserver() {
      public void update(IObservable var1, Object var2) {
         DanteController.this.handleInterfaceUpdateEvent(DanteController.this.mCmmClient);
      }
   };
   private ControllerModel mControllerModel;
   private int mState = 0;
   private Queue<Pair<URI, Integer>> mPendingUrls = new LinkedList();
   private static UrlSchemeHandler sHandler;
   private ICmmClientListener mCmmListener;
   private DdhController mDdhController;

   public void initUI(IJavaEnvironment var1) {
      Icon var2 = UIManager.getIcon("OptionPane.questionIcon");

      try {
         InputStream var3 = this.getClass().getResourceAsStream("/fonts/DanteSansGUI-Text.otf");
         Font var4 = Font.createFont(0, var3);
         GraphicsEnvironment.getLocalGraphicsEnvironment().registerFont(var4);
         InputStream var5 = this.getClass().getResourceAsStream("/fonts/DanteSansGUI-Display.otf");
         Font var6 = Font.createFont(0, var5);
         GraphicsEnvironment.getLocalGraphicsEnvironment().registerFont(var6);
      } catch (IOException | FontFormatException var7) {
         var7.printStackTrace();
      }

      ThemeSelector.init();
      Icon var8 = UIManager.getIcon("OptionPane.questionIcon");
      if (var8 == null) {
         UIManager.put("OptionPane.questionIcon", var2);
      }

   }

   public ControllerConfig getConfig() {
      return this.mConfig;
   }

   public MenuFrame getControllerFrame() {
      return this.mControllerFrame;
   }

   public List<DeviceFrame> getDeviceFrames() {
      return this.mDeviceFrames.getAllFrames();
   }

   public void createApplication() throws AudinateException {
      this.mApplication = new ControllerApplication(this, this.mStartTime);
      this.mApplication.init();
   }

   public void suggestNoValidInterfaces() {
      this.mApplication.suggestNoValidInterfaces(this.mControllerFrame);
   }

   public void suggestReboot(String var1) {
      this.mApplication.showRebootWarning(this.mControllerFrame, var1);
   }

   public void createModel() throws AudinateException {
      this.mLogicModel = new LogicModel(this.mApplication);
      this.mLogic = new ControllerLogic(this.mApplication, this.mLogicModel, this, this);
      this.mLogicModel.setRouterController(this.mLogic.getRouterController());
      this.mControllerModel = new ControllerModel(this.mApplication, this.mApplication.getControllerConfig(), this.mLogicModel.getUnfilteredDevices(), this.mLogicModel.getRtpFlows(), this.mLogicModel);
      this.mConfig.setDomainHandler(this.mLogicModel.getDomainHandler());
   }

   public void createView() throws AudinateException {
      this.mDdhController = new DdhController(this.mApplication, this.mLogicModel.getDomainHandler());
      ImageLibrary var1 = this.mApplication.getImageLibrary();
      ThemedImage[] var2 = new ThemedImage[]{null, null, var1.getImage("ERROR_32"), var1.getImage("WARNING_32"), var1.getImage("INFORMATION_32")};
      this.mControllerFrame = new ControllerFrame(this.mApplication, this.mLogic.getRouterController(), this.mLogic.getConmonController(), this, this.mLogicModel, this.mLogic.getPolls(), this.mControllerModel.getLatencyScheme(), this.mDdhController, this.mControllerModel, var2);
      this.mControllerFrame.setLogic(this.mLogic);
      this.mControllerFrame.canAddListeners();
      this.mControllerFrame.createCmmConfigDialog();
      this.mDeviceFrames = new DeviceFrames(this.mApplication, this.mControllerModel, this.mLogic, this, this.mLogicModel.getMillauLockClient(), this.mControllerFrame, var2);
      this.mDeviceFrames.localise(this.mBundle);
   }

   private void startSystem() throws AudinateException {
      IJavaEnvironment var1 = JavaEnvironment.getInstance();
      this.mControllerFrame.pack();
      GraphicsEnvironment var2 = GraphicsEnvironment.getLocalGraphicsEnvironment();
      Rectangle var3 = var2.getMaximumWindowBounds();
      Dimension var4 = new Dimension();
      var4.width = Math.min(var3.width, 1400);
      var4.height = Math.min(var3.height, 900);
      this.mControllerFrame.setSize(var4);
      ScreenUtils.centerOnScreen(this.mControllerFrame, (Container)null);
      this.mControllerFrame.setVisible(true);
      this.mApplication.start();
      this.mLogicModel.start();
      this.mLogic.start();
      String var5 = "DanteControllerAgent";
      String var6 = "DanteGateway";
      int var7 = this.mApplication.getPlatform().getPlatformType();
      if (var7 == 0) {
         var5 = var5 + ".exe";
         var6 = var6 + ".exe";
      }

      String var8 = System.getProperty("dc.agent.path");
      if (var8 != null && !"".equals(var8)) {
         var5 = resolvePath(var1, var8);
      }

      String var9 = System.getProperty("dc.translator.path");
      if (var9 != null && !"".equals(var9)) {
         String var10000 = resolvePath(var1, var9);
         var6 = var10000 + "/" + var6;
      }

      File var10 = new File(var6);
      if (var10.exists() && var10.canExecute()) {
         this.mTranslatorPath = var6;
      } else {
         this.mApplication.getDebugLog().log(4, (Log.ISource)null, "Argo translator executable not found: " + var6);
      }

      File var11 = new File(var5);
      if (var11.exists() && var11.canExecute()) {
         this.mAgentPath = var5;
         this.mPluginSearchPaths = new LinkedList();
         String var12 = this.getConfig().getPluginPath();
         if (var12 != null && !var12.isEmpty()) {
            this.mPluginSearchPaths.add(var12);
         }

         String var13 = "./plugins";
         String var14 = System.getProperty("dc.plugin.path");
         if (var14 != null) {
            var13 = resolvePath(var1, var14);
         }

         this.mPluginSearchPaths.add(var13);
      }

      this.mApplication.getSchedulableTimers().createRelativeTimer(this, 1000L, "checkProcessesRunning");
      this.mCmmClients = this.mLogicModel.getCmmClients();
      this.mCmmClients.addObserver(this.mCmmClientsObserver);
      this.setCmmClient(this.mCmmClients.getSelectedClient());
      this.mArgoInterface = null;
      this.mApplication.getNetworkInterfaceManager().addObserver(this.mInterfaceManagerObserver);
   }

   private static String resolvePath(IJavaEnvironment var0, String var1) {
      return Paths.get(var1).toAbsolutePath().normalize().toString();
   }

   public void startInGuiThread() {
      SwingUtilities.invokeLater(() -> {
         this.start();
      });
   }

   public void stopInGuiThread() {
      SwingUtilities.invokeLater(() -> {
         this.stop();
      });
   }

   public void start() {
      this.mState = 4;

      try {
         this.createApplication();
         String[] var1 = this.mApplication.getResourceLocations();
         Image var2 = ImageLibrary.loadImageHelper("images", "splashscreen.png", var1).getImage();
         SplashScreen.show(new ImageIcon(var2), 5000, () -> {
            try {
               if (this.mApplication.getControllerConfig().isPreloadingBrowser()) {
                  this.mApplication.getBrowserEngine();
               }
            } catch (Throwable var2) {
               var2.printStackTrace();
               this.mApplication.getDebugLog().logThrowable(4, (Log.ISource)null, var2.getMessage(), var2);
               System.exit(1);
            }

         }, () -> {
            try {
               this.createModel();
               this.createView();
               this.startSystem();
            } catch (Throwable var2) {
               var2.printStackTrace();
               this.mApplication.getDebugLog().logThrowable(4, (Log.ISource)null, var2.getMessage(), var2);
               System.exit(1);
            }

            String var1 = this.mConfig.getLanguage();
            this.mApplication.getAnalyticsReporter().submitEvent("APPLICATION_LAUNCHED", JsonUtil.toJson(Map.of("languageCfg", var1 != null ? JsonUtil.toJson(var1) : JsonNull.VALUE, "language", JsonUtil.toJson(this.mConfig.getStringsBundle().getString("i18n.language.code")))));
            new IdleStateManager(new IdleStateListener() {
               public void onIdle(AWTEvent var1) {
                  DanteController.this.mApplication.getAnalyticsReporter().submitEvent("USER_IS_IDLE", JsonUtil.toJson(Map.of("isIdleAfter", JsonUtil.toJson(300000))));
               }

               public void onWorking(AWTEvent var1, long var2) {
                  if (var2 > 0L) {
                     DanteController.this.mApplication.getAnalyticsReporter().submitEvent("USER_IS_ONLINE", JsonUtil.toJson(Map.of("wasIdleFor", JsonUtil.toJson(var2))));
                  }

               }
            });
            this.mState = 2;
            this.mCmmListener = new ICmmClientListener() {
               public void cmmOptionsChanged(ICmmClient var1) {
               }

               public void cmmStateChanged(ICmmClient var1) {
                  if (DanteController.this.mState == 2) {
                     DanteController.this.mState = 3;
                     SwingUtilities.invokeLater(() -> {
                        DanteController.this.processUrls();
                     });
                  }

                  DanteController.this.handleInterfaceUpdateEvent(var1);
               }
            };
            this.mCmmClient.addCmmClientListener(this.mCmmListener);
            this.mDdhController.addEventListener((var1x) -> {
               if (var1x.getType() == 0) {
                  this.mApplication.getDebugLog().log(6, (Log.ISource)null, "Explicit user logout - removing persisted credentials.");
                  this.mApplication.getEncryptedStore().remove("url");
               } else if (var1x.getType() == 1) {
                  this.mApplication.getDebugLog().log(6, (Log.ISource)null, "Connection to the Domain Manager lost.");
                  String var2 = "<HTML>" + ResourceBundleUtils.getString(this.mBundle, "domainHandler.errors.connectionLost") + "</HTML>";
                  this.mControllerFrame.hideReconnectDialog();
                  this.setEnabled(true);
                  this.mApplication.getDialogs().showMessage(this.mControllerFrame, var2);
               } else if (var1x.getType() == 2) {
                  this.mApplication.getDebugLog().log(4, (Log.ISource)null, "Connection to the Domain Manager lost. Reconnecting ...");
                  this.setEnabled(false);
                  this.mControllerFrame.showReconnectDialog((var1) -> {
                     this.mControllerFrame.hideReconnectDialog();
                     this.setEnabled(true);
                     this.mControllerFrame.setAlwaysOnTop(true);
                     this.mControllerFrame.setAlwaysOnTop(false);
                     this.mDdhController.cancelReconnect();
                  });
               } else if (var1x.getType() == 3) {
                  this.mApplication.getDebugLog().log(4, (Log.ISource)null, "Connection to the Domain Manager reestablished.");
                  this.mControllerFrame.hideReconnectDialog();
                  this.setEnabled(true);
               }

            });
            if (this.mApplication.getEncryptedStore().containsKey("url") && this.mPendingUrls.isEmpty()) {
               this.mApplication.getDebugLog().log(6, (Log.ISource)null, "Scheduling persisted credentials for processing.");
               this.mPendingUrls.add(new Pair(URI.create(this.mApplication.getEncryptedStore().get("url").asString().getValue()), SOURCE_PERSISTED));
            }

         });
      } catch (Throwable var3) {
         var3.printStackTrace();
         this.mApplication.getDebugLog().logThrowable(4, (Log.ISource)null, var3.getMessage(), var3);
         System.exit(1);
      }

   }

   private void setEnabled(boolean var1) {
      this.mControllerFrame.setEnabled(var1);
      this.mDeviceFrames.setEnabled(var1);
   }

   public void stop() {
      this.mCmmClient.removeCmmClientListener(this.mCmmListener);
      this.mState = 1;
      this.mDeviceFrames.closeAllFrames();
      this.mControllerFrame.closeFrame();
      this.mControllerFrame.setVisible(false);
      this.mControllerFrame.dispose();
      this.mLogic.stop();
      this.mLogicModel.stop();
      this.mApplication.stop();
      this.stopAgent();
      this.mApplication.getAnalyticsReporter().submitEvent("APPLICATION_TERMINATED", (JsonObject)null);
      this.mApplication.getAnalyticsReporter().waitPending();
      this.mState = 0;
      System.exit(0);
   }

   public void showDevice(String var1) {
      this.mDeviceFrames.getFrame(var1);
   }

   public void showDeviceComponent(String var1, int var2, int var3) {
      DeviceFrame var4 = this.mDeviceFrames.getFrame(var1);
      var4.showComponent(var2, var3, -1);
   }

   public void showDeviceComponent(String var1, int var2, int var3, int var4) {
      DeviceFrame var5 = this.mDeviceFrames.getFrame(var1);
      var5.showComponent(var2, var3, var4);
   }

   public void showDeviceStat(String var1, int var2, int var3) {
      DeviceFrame var4 = this.mDeviceFrames.getFrame(var1);
      var4.showStat(var2, var3);
   }

   public void chooseInterfaces() {
      this.mControllerFrame.chooseInterfaces();
   }

   public void suggestInvalidInterfaceConfiguration() {
      this.mControllerFrame.suggestInvalidInterfaceConfiguration();
   }

   public static void dumpProperties() {
      System.err.println("Properties:");
      Properties var0 = System.getProperties();
      Iterator var1 = var0.keySet().iterator();

      while(var1.hasNext()) {
         Object var2 = var1.next();
         Object var3 = var0.get(var2);
         System.err.println("" + var2 + "=" + var3);
      }

   }

   public static void main(String[] var0) throws Exception {
      DanteController var1 = createController(var0);
      SwingUtilities.invokeAndWait(() -> {
         var1.start();
      });
   }

   public static DanteController createController(String[] var0) throws Exception {
      if (!Preconditions.checkPreconditions()) {
         System.exit(0);
      }

      IJavaEnvironment var1 = JavaEnvironment.getInstance();
      Util.suppressLibrary();
      DanteDapi.suppressLibrary();

      try {
         if (var1.isOsx()) {
            String var2 = "./libDanteController.dylib";
            String var3 = System.getProperty("dc.dylib.path");
            if (var3 != null) {
               var2 = resolvePath(var1, var3);
            }

            System.out.println("lib: " + var2);
            File var4 = new File(var2);
            if (var4.exists()) {
               System.load(var2);
            }
         } else {
            System.loadLibrary("DanteController");
         }
      } catch (Exception var6) {
         System.out.println("Exception.Msg: " + var6.toString());
         var6.printStackTrace();
         System.exit(0);
      } catch (Error var7) {
         System.out.println("Error.Msg: " + var7.toString());
         var7.printStackTrace();
         System.exit(0);
      }

      AudinateUtils.verifyEnums();
      if (var0.length > 0 && var0[0].equalsIgnoreCase("-convert")) {
         String[] var8 = new String[var0.length - 1];
         System.arraycopy(var0, 1, var8, 0, var8.length);
         LegacyUtils.main(var8);
         System.exit(0);
      }

      DanteController var9 = new DanteController();
      OpenURIHandler var10 = (var1x) -> {
         SwingUtilities.invokeLater(() -> {
            var9.handleUrl(var1x.getURI());
         });
      };
      Desktop var11 = Desktop.getDesktop();
      if (var11.isSupported(Action.APP_OPEN_URI)) {
         var11.setOpenURIHandler(var10);
      } else {
         sHandler = new UrlSchemeHandler("dantecontroller");
         if (var0.length > 1 && "--open-url".equalsIgnoreCase(var0[0])) {
            if (sHandler.submitUrl(var0[1])) {
               var10.openURI(new OpenURIEvent(sHandler.getLastUrl()));
               String[] var5 = new String[var0.length - 2];
               System.arraycopy(var0, 2, var5, 0, var5.length);
               var0 = var5;
            } else {
               System.exit(0);
            }
         }

         sHandler.startListening(var10);
      }

      var9.mConfig.setAutoThemeFn(() -> {
         return var9.mApplication.getPlatform().isDarkModeEnabled() ? "dark" : "light";
      });
      var9.mConfig.parseArgs(var0);
      if (var9.mConfig.getStartDelay() > 0) {
         System.err.println("Starting start delay");
         Thread.sleep((long)(var9.mConfig.getStartDelay() * 1000));
         System.err.println("Finished start delay");
      }

      var9.initUI(var1);
      return var9;
   }

   private void handleUrl(URI var1) {
      this.mPendingUrls.add(new Pair(var1, SOURCE_URL_HANDLER));
      if (this.mState == 3) {
         this.processUrls();
      }

   }

   private void processUrls() {
      if (!this.mPendingUrls.isEmpty()) {
         this.mControllerFrame.setState(0);
         this.mControllerFrame.setAlwaysOnTop(true);
         this.mControllerFrame.setAlwaysOnTop(false);
      }

      while(!this.mPendingUrls.isEmpty()) {
         Pair var1 = (Pair)this.mPendingUrls.remove();
         this.processUrl((URI)var1.getFirst(), (Integer)var1.getSecond());
      }

   }

   private void processUrl(URI var1, Integer var2) {
      Pattern var3 = Pattern.compile("^dantecontroller:(?:(?:([^:@]*):?([^:@]*))@)?((?:[^?#\\/]*\\/)*[^?#\\/]+)\\/?(?:\\?((?:[^&#]*&)*(?:[^&#]*)))?(?:\\#((?:[^&#]*)))?$");
      Matcher var4 = var3.matcher(var1.toString());
      if (var4.matches()) {
         String var5 = this.urlDecodeSafe(var4.group(3));
         String var6 = var4.group(4);
         Map var7 = var6 != null ? (Map)Arrays.asList(var6.split("&")).stream().map((var0) -> {
            return var0.split("=");
         }).collect(Collectors.toMap((var1x) -> {
            return var1x.length > 0 ? this.urlDecodeSafe(var1x[0]).toLowerCase() : null;
         }, (var1x) -> {
            return var1x.length > 1 ? this.urlDecodeSafe(var1x[1]) : null;
         })) : Collections.emptyMap();
         this.mApplication.getDebugLog().log(8, (Log.ISource)null, "Url Path: " + var5);
         this.mApplication.getDebugLog().log(8, (Log.ISource)null, "Url Query Params: " + var7);
         if (var7.containsKey("minversion")) {
            Version var8 = new Version((String)var7.get("minversion"), 3);
            if (SafeEquals.safeCompare(var8, IDanteControllerVersion.VERSION) > 0) {
               this.mApplication.getDialogs().showError(this.mControllerFrame, MessageFormat.format(ResourceBundleUtils.getString(this.mBundle, "errors.applicationUrlNotSupported"), var8));
               return;
            }
         }

         boolean var14 = false;
         if (var7.containsKey("host") && var7.containsKey("port")) {
            String var9 = (String)var7.get("host");
            int var10 = Integer.parseInt((String)var7.get("port"));
            String var11 = (String)var7.get("user");
            if (var7.containsKey("user") && var7.containsKey("pass")) {
               this.mControllerFrame.initDomainHandler(var11, var9, String.valueOf(var10));
               this.mDdhController.setSourceWindow(this.mControllerFrame);
               this.mDdhController.setDomainManagerAddr(var9, var10);
               var14 = true;
               this.mDdhController.loginWithUserPass(var11, (String)var7.get("pass"), (String)var7.get("domain"), (var4x) -> {
                  if (var4x == 0) {
                     this.processPath(var1, var5, var7);
                  }

               });
            } else if (var7.containsKey("token")) {
               this.mControllerFrame.initDomainHandler(var11, var9, String.valueOf(var10));
               this.mDdhController.setSourceWindow(this.mControllerFrame);
               this.mDdhController.setDomainManagerAddr(var9, var10);
               boolean var12 = var7.containsKey("persist") && Boolean.parseBoolean((String)var7.get("persist"));
               var14 = true;
               boolean var13 = var12 || SOURCE_PERSISTED.equals(var2);
               this.mDdhController.loginWithToken(var11, (String)var7.get("token"), (String)var7.get("domain"), var13, (var6x) -> {
                  if (var6x == 0) {
                     if (var12 && !SOURCE_PERSISTED.equals(var2)) {
                        this.mApplication.getDebugLog().log(6, (Log.ISource)null, "Processed application url with persist - persisting credentials.");
                        this.mApplication.getEncryptedStore().put("url", JsonUtil.toJson(this.buildCustomUrl("default", this.pick(var7, "host", "port", "user", "token"))));
                     }

                     this.processPath(var1, var5, var7);
                  } else if (SOURCE_PERSISTED.equals(var2)) {
                     ArrayList var7x = new ArrayList();
                     boolean var8 = false;
                     switch (var6x) {
                        case 42:
                           var7x.add(ResourceBundleUtils.getString(this.mBundle, "domainHandler.errors.notConnectedMaybeBadToken"));
                           var8 = true;
                           break;
                        default:
                           var7x.add(ResourceBundleUtils.getString(this.mBundle, "domainHandler.errors.couldNotRestoreLogin"));
                           DdhController var10001 = this.mDdhController;
                           var7x.add("<em>" + var10001.getErrorString(var6x) + "</em>");
                           var7x.add(ResourceBundleUtils.getString(this.mBundle, "domainHandler.confirmClearPersistedCredentials"));
                     }

                     Stream var10000 = var7x.stream();
                     String var9 = "<HTML>" + (String)var10000.collect(Collectors.joining("<br/><br/>")) + "</HTML>";
                     if (!var8) {
                        if (!this.mApplication.getDialogs().queryYesNo(this.mControllerFrame, this.mControllerFrame.getTitle(), var9)) {
                           this.mApplication.getDebugLog().log(4, (Log.ISource)null, "Removing persisted credentials after failed login..");
                           this.mApplication.getEncryptedStore().remove("url");
                        }
                     } else {
                        this.mApplication.getDialogs().showMessage(this.mControllerFrame, var9);
                        this.mApplication.getDebugLog().log(4, (Log.ISource)null, "Removing persisted credentials due to likely expiry.");
                        this.mApplication.getEncryptedStore().remove("url");
                     }
                  } else {
                     this.mDdhController.showError(this.mControllerFrame, var6x);
                  }

               });
            }
         } else if (var7.containsKey("domain")) {
            this.mControllerFrame.initDomainHandler((String)null, (String)null, (String)null);
            this.mDdhController.setActiveDomainByName((String)var7.get("domain"));
         }

         if (!var14) {
            this.processPath(var1, var5, var7);
         } else if (!SOURCE_PERSISTED.equals(var2) && this.mApplication.getEncryptedStore().containsKey("url")) {
            this.mApplication.getDebugLog().log(6, (Log.ISource)null, "Processed new application url, removing existing persisted credentials.");
            this.mApplication.getEncryptedStore().remove("url");
         }
      }

   }

   @SafeVarargs
   private <K, V> Map<K, V> pick(Map<K, V> var1, K... var2) {
      HashMap var3 = new HashMap();
      var3.putAll(var1);
      var3.keySet().retainAll(Arrays.asList(var2));
      return var3;
   }

   private String buildCustomUrl(String var1, Map<String, String> var2) {
      StringBuilder var3 = new StringBuilder();
      var3.append("dantecontroller:");

      try {
         var3.append(URLEncoder.encode(var1, "UTF-8"));
         if (!var2.isEmpty()) {
            var3.append('?');
            var3.append((String)var2.entrySet().stream().map((var0) -> {
               try {
                  String var10000 = URLEncoder.encode((String)var0.getKey(), "UTF-8");
                  return var10000 + "=" + URLEncoder.encode((String)var0.getValue(), "UTF-8");
               } catch (UnsupportedEncodingException var2) {
                  var2.printStackTrace();
                  return "";
               }
            }).collect(Collectors.joining("&")));
         }
      } catch (UnsupportedEncodingException var5) {
         var5.printStackTrace();
      }

      return var3.toString();
   }

   private boolean processPath(URI var1, String var2, Map<String, String> var3) {
      if ("routing".equals(var2)) {
         this.mControllerFrame.setActiveTab(0);
         return true;
      } else if ("devices".equals(var2)) {
         this.mControllerFrame.setActiveTab(1);
         return true;
      } else if ("devices-clocking".equals(var2)) {
         this.mControllerFrame.setActiveTab(2);
         return true;
      } else if ("devices-network".equals(var2)) {
         this.mControllerFrame.setActiveTab(3);
         return true;
      } else if ("events".equals(var2)) {
         this.mControllerFrame.setActiveTab(4);
         return true;
      } else if ("activator".equals(var2)) {
         this.mControllerFrame.triggerActivatorAction();
         return true;
      } else if ("updater".equals(var2)) {
         this.mControllerFrame.triggerUpdaterAction();
         return true;
      } else if ("set-domain-manager".equals(var2)) {
         this.mControllerFrame.openEnrolConfigDialog((String)var3.get("enroldomain"));
         return true;
      } else if ("login-domain-manager".equals(var2)) {
         this.mControllerFrame.openDomainHandler((String)null, (String)null, (String)null);
         return true;
      } else if ("default".equals(var2)) {
         return true;
      } else {
         this.mApplication.getDebugLog().log(3, (Log.ISource)null, "Unhandled url: \"" + var1.toString() + "\"");
         return false;
      }
   }

   private String urlDecodeSafe(String var1) {
      try {
         return URLDecoder.decode(var1, "UTF-8");
      } catch (UnsupportedEncodingException var3) {
         return var1;
      }
   }

   public void showEvents() {
      this.mControllerFrame.setActiveTab(4);
   }

   public void showClockMonitoring() {
      this.mControllerFrame.showClockMonitoring();
   }

   public void addClockMonitoringObserver(IObserver var1) {
      this.mLogic.addClockMonitoringObserver(var1);
   }

   public void addClockMonitoringWindowListener(WindowListener var1) {
      this.mControllerFrame.addClockMonitoringWindowListener(var1);
   }

   public void addClockMonitoringComponentListener(ComponentListener var1) {
      this.mControllerFrame.addClockMonitoringComponentListener(var1);
   }

   public void clearClockEventsForAllDevices() {
      this.mLogic.clearClockEventsForAllDevices();
   }

   public void addFilteredNotificationByLogLevelListener(IObserver var1) {
      this.mControllerFrame.addFilteredNotificationByLogLevelListener(var1);
   }

   public void doShowSystemInformation(JFrame var1) {
      this.mControllerFrame.doShowSystemInformation(var1);
   }

   public void showTxRxLocked(JFrame var1) {
      JOptionPane.showOptionDialog(var1, new JLabel("<html>Transmitter and Receiver are locked.<br/>Unlock these devices to make changes.</html>", 0), "Warning", -1, -1, (Icon)null, new String[]{"Close"}, "default");
   }

   public void showRxOnlyUnlocked(JFrame var1, IDevice var2) {
      int var3 = JOptionPane.showOptionDialog(var1, new JLabel("<html>The Receiver is locked. Unlock the device to make changes.</html>", 0), "Warning", -1, -1, (Icon)null, new String[]{"Unlock Receiver", "Close"}, "default");
      if (var3 == 0) {
         this.showLockDialogModal(var1, var2);
      }

   }

   public boolean showTxOnlyUnlocked(JFrame var1) {
      String var2 = this.mApplication.getPreferences().get("WARN_TX_LOCKED_SUBSCRIPTIONS", "warn");
      boolean var3 = !"ignore".equals(var2);
      if (var3) {
         JCheckBox var4 = new JCheckBox("Don't ask me again.");
         JLabel var5 = new JLabel("<html>Transmitter is locked.  Only subscriptions present at <br/>the time the transmitter was locked may be established.</html>", 0);
         Object[] var6 = new Object[]{var5, var4};
         int var7 = JOptionPane.showOptionDialog(var1, var6, "Warning", 2, -1, (Icon)null, new String[]{"Subscribe", "Cancel"}, "default");
         this.mApplication.getPreferences().put("WARN_TX_LOCKED_SUBSCRIPTIONS", !var4.isSelected() ? "warn" : "ignore");
         return var7 == 0;
      } else {
         return true;
      }
   }

   public void showLockDialogModal(JFrame var1, IDevice var2) {
      if (!DeviceUtils.isLocked(var2)) {
         boolean var3 = StaleAudioConnectionsDialog.checkStaleConnections(var1, this.mControllerModel.getUnfilteredDevices(), var2.getName(), this.mLogic.getRouterController(), this.mApplication.getSchedulableTimers());
         if (!var3) {
            return;
         }
      }

      try {
         ImageLibrary var4 = this.mApplication.getImageLibrary();
         ThemedImage[] var5 = new ThemedImage[]{var4.getImage("TICK_30"), var4.getImage("CROSS_30")};
         MillauLockDialog var7 = new MillauLockDialog(var1, var5, this.mLogicModel.getMillauLockClient(), this.mApplication);
         var7.localise(this.mBundle);
         var7.setModal(true);
         var7.setDevice(var2);
         var7.pack();
         var7.setLocationRelativeTo(var1);
         var7.setVisible(true);
      } catch (AudinateException var6) {
         var6.printStackTrace();
      }

   }

   private void updateAgentState() {
      if (this.mCmmClient != null) {
         if (this.mCmmClient.getConnectionState() == 2) {
            CmmState var1 = this.mCmmClient.getSystemState();
            if (var1 != null) {
               if (this.mAgentProcess != null) {
                  this.stopAgent();
               }

               this.startAgent();
            }
         }
      }
   }

   private long getPid() {
      return ProcessHandle.current().pid();
   }

   public void timerFired(ISchedulableTimer var1) {
      if (this.mAgentProcess != null) {
         this.handleAgentTimer();
      }

      if (this.mTranslatorProcess != null) {
         this.handleTranslatorTimer();
      }

      this.mApplication.getSchedulableTimers().createRelativeTimer(this, 1000L, "checkProcessesRunning");
   }

   private void handleAgentTimer() {
      if (!this.mAgentProcess.isAlive()) {
         this.mAgentProcess = null;
         if (this.mAgentRestartCount < 5) {
            ++this.mAgentRestartCount;
            this.startAgent();
            this.mApplication.getDebugLog().log(7, (Log.ISource)null, "Agent restart...");
         } else {
            JOptionPane.showMessageDialog(this.mControllerFrame, "Error: DanteControllerAgent can't be restarted.");
         }
      }

   }

   private void handleTranslatorTimer() {
      if (!this.mTranslatorProcess.isAlive()) {
         this.mTranslatorProcess = null;
         this.startTranslator();
         this.mApplication.getDebugLog().log(7, (Log.ISource)null, "Translator restart...");
      }

   }

   private void startAgent() {
      if (this.mAgentProcess == null) {
         ArrayList var1 = new ArrayList();
         long var2 = this.getPid();
         if (this.mAgentPath == null) {
            JOptionPane.showMessageDialog((Component)null, "Could not find DanteControllerAgent executable.");
         } else {
            var1.add(this.mAgentPath);
            var1.add("--pid=" + var2);
            if (this.mAgentRemoteComponentPort == 0) {
               this.mAgentRemoteComponentPort = this.mApplication.getJniEnvironment().createRemoteComponent();
            }

            var1.add("--conmon-port=" + this.mAgentRemoteComponentPort);
            Boolean var4 = this.mCmmClients.getSelection().isGlobal();
            ControllerConfig var5 = this.getConfig();
            int var6 = var4 ? 0 : var5.getConmonServerPort(false);
            var1.add("--unmanaged-conmon-port=" + var6);
            Iterator var8 = this.mPluginSearchPaths.iterator();

            String var10;
            while(var8.hasNext()) {
               var10 = (String)var8.next();
               var1.add("--library-path=" + var10);
            }

            Log var10000 = this.mApplication.getDebugLog();
            Stream var10003 = var1.stream().map((var0) -> {
               return var0.contains(" ") ? "'" + var0 + "'" : var0;
            });
            var10000.log(6, (Log.ISource)null, "Launching " + (String)var10003.collect(Collectors.joining(" ")));
            ProcessBuilder var9 = new ProcessBuilder(var1);
            var9.environment().put("LOG_LEVEL", Log.LEVELS[0]);
            var10 = this.mApplication.getAgentLogFilename();
            if (var10 != null) {
               var9.environment().put("LOG_LEVEL", Log.LEVELS[this.mConfig.getAgentLogLevel()]);
               var9.environment().put("LOG_FILE", var10);
            }

            try {
               var9.redirectOutput(Redirect.PIPE);
               var9.redirectErrorStream(true);
               this.mAgentProcess = var9.start();
               AgentOutputThread var11 = new AgentOutputThread(this, this.mAgentProcess.getInputStream(), "OUTPUT");
               var11.start();
            } catch (Exception var7) {
               JOptionPane.showMessageDialog((Component)null, "Error starting agent: " + var7.toString());
            }
         }
      }

   }

   public boolean stopAgent() {
      if (this.mAgentProcess != null) {
         this.mAgentProcess.destroyForcibly();
         this.mAgentProcess = null;
      }

      if (this.mAgentRemoteComponentPort != 0) {
         this.mApplication.getJniEnvironment().destroyRemoteComponent(this.mAgentRemoteComponentPort);
         this.mAgentRemoteComponentPort = 0;
      }

      return true;
   }

   private void startTranslator() {
      if (this.mTranslatorProcess == null && this.mTranslatorPath != null) {
         ArrayList var1 = new ArrayList();
         var1.add(this.mTranslatorPath);
         if (this.mArgoInterface != null && this.mArgoInterface.getIpAddress() != null && this.mArgoInterface.isLinkUp()) {
            var1.add("--argo-addr=" + this.mArgoInterface.getIpAddress());
            var1.add("--argo-port=" + this.mConfig.quicArgoPort());
            var1.add("--graphql-port=" + this.mConfig.graphQlPort());
            var1.add("--require-stdin");
            Log var10000 = this.mApplication.getDebugLog();
            Stream var10003 = var1.stream().map((var0) -> {
               return var0.contains(" ") ? "'" + var0 + "'" : var0;
            });
            var10000.log(6, (Log.ISource)null, "Launching " + (String)var10003.collect(Collectors.joining(" ")));
            ProcessBuilder var2 = new ProcessBuilder(var1);

            try {
               var2.redirectOutput(Redirect.PIPE);
               var2.redirectErrorStream(true);
               this.mTranslatorProcess = var2.start();
               TranslatorOutputThread var3 = new TranslatorOutputThread(this, this.mTranslatorProcess.getInputStream(), "OUTPUT");
               var3.start();
            } catch (Exception var4) {
               this.mApplication.getDebugLog().log(6, (Log.ISource)null, String.format("Error starting the Argo translator process: %s", var4.getMessage()));
            }

         } else {
            this.mApplication.getDebugLog().log(4, (Log.ISource)null, "Error starting the Argo translator process - Argo interface link is down or ip address is not available");
         }
      }
   }

   public int getWebsocketPort() {
      return this.mAgentWebsocketPort;
   }

   public int getWebserverPort() {
      return this.mAgentWebserverPort;
   }

   public void cmmOptionsChanged(ICmmClient var1) {
      this.updateAgentState();
   }

   public void cmmStateChanged(ICmmClient var1) {
      this.updateAgentState();
   }

   private void setCmmClient(ICmmClient var1) {
      if (this.mCmmClient != null) {
         this.mCmmClient.removeCmmClientListener(this);
         this.mCmmClient = null;
      }

      this.mCmmClient = var1;
      if (this.mCmmClient != null) {
         this.mCmmClient.addCmmClientListener(this);
      }

      this.updateAgentState();
   }

   public void setLegacyAssociatedDevicesShown(boolean var1) {
      this.mControllerModel.setLegacyAssociatedDevicesShown(var1);
   }

   public void localise(ResourceBundle var1) {
      if (var1 != null) {
         this.mBundle = var1;
      }

      if (this.mControllerFrame != null) {
         this.mControllerFrame.localise(var1);
      }

      if (this.mDeviceFrames != null) {
         this.mDeviceFrames.localise(var1);
      }

   }

   private void handleInterfaceUpdateEvent(ICmmClient var1) {
      CmmConfig var2 = var1.getSystemState().getCurrentConfig();
      NetworkInterface var3 = this.mArgoInterface;
      if (var2.numInterfaces() > 0) {
         CmmInterface var4 = var2.interfaceAtIndex(0);
         this.mArgoInterface = this.mApplication.getNetworkInterfaceManager().interfaceWithName(var4.getName());
         if (this.mArgoInterface != null) {
            if (var3 == null || var3.getIpAddress() != this.mArgoInterface.getIpAddress() || this.mTranslatorProcess == null || !this.mArgoInterface.isLinkUp()) {
               if (this.mTranslatorProcess != null) {
                  this.mTranslatorProcess.destroy();
               }

               if (this.mArgoInterface.isLinkUp()) {
                  this.startTranslator();
               }

            }
         }
      }
   }

   private class AgentOutputThread extends Thread {
      private InputStream is;
      private Boolean warnUser;

      private AgentOutputThread(DanteController var2, InputStream var3, String var4) {
         this.is = var3;
         this.warnUser = true;
      }

      public void run() {
         try {
            Gson var1 = new Gson();
            InputStreamReader var2 = new InputStreamReader(this.is);
            BufferedReader var3 = new BufferedReader(var2);
            String var4 = null;

            while(true) {
               do {
                  do {
                     if ((var4 = var3.readLine()) == null) {
                        return;
                     }
                  } while(var4.length() <= 0);
               } while(var4.charAt(0) != '{');

               try {
                  Msg var5 = (Msg)var1.fromJson(var4, Msg.class);
                  String var6;
                  if (var5.service != null && var5.service.equals("websocket")) {
                     DanteController.this.mAgentWebsocketPort = var5.port;
                     var6 = "Agent Websocket Port is " + DanteController.this.mAgentWebsocketPort;
                     DanteController.this.mApplication.getDebugLog().log(7, (Log.ISource)null, var6);
                     if (this.warnUser && DanteController.this.mAgentWebsocketPort == 0) {
                        this.warnUser = false;
                        JOptionPane.showMessageDialog((Component)null, "Error: DanteControllerAgent's websocket port is 0.");
                     }
                  } else if (var5.service != null && var5.service.equals("webserver")) {
                     DanteController.this.mAgentWebserverPort = var5.port;
                     var6 = "Agent Webserver Port is " + DanteController.this.mAgentWebserverPort;
                     DanteController.this.mApplication.getDebugLog().log(7, (Log.ISource)null, var6);
                     if (this.warnUser && DanteController.this.mAgentWebserverPort == 0) {
                        this.warnUser = false;
                        JOptionPane.showMessageDialog((Component)null, "Error: DanteControllerAgent's web server port is 0.");
                     }
                  }
               } catch (JsonSyntaxException var7) {
                  var7.printStackTrace();
               }
            }
         } catch (IOException var8) {
            var8.printStackTrace();
         }
      }

      private class Msg {
         private String service;
         private int port;
      }
   }

   private class TranslatorOutputThread extends Thread {
      private InputStream is;
      private FileOutputStream mLogFile;

      private TranslatorOutputThread(DanteController var2, InputStream var3, String var4) {
         this.is = var3;

         try {
            File var5 = new File(DanteController.this.mApplication.getTranslatorLogFilename());
            var5.createNewFile();
            this.mLogFile = new FileOutputStream(var5, true);
         } catch (IOException var6) {
            DanteController.this.mApplication.getDebugLog().log(4, (Log.ISource)null, String.format("Could not create translator log file: %s. Defaulting to stdout.", var6.getMessage()));
            this.mLogFile = null;
         }

      }

      protected void finalize() {
         try {
            this.mLogFile.close();
         } catch (IOException var2) {
            var2.printStackTrace();
         }

      }

      public void run() {
         try {
            InputStreamReader var1 = new InputStreamReader(this.is);
            BufferedReader var2 = new BufferedReader(var1);
            String var3 = null;

            while((var3 = var2.readLine()) != null) {
               if (this.mLogFile != null) {
                  String var4 = LocalDateTime.now().toString();
                  String var5 = String.format("%s: %s\n", var4, var3);
                  this.mLogFile.write(var5.getBytes());
               } else {
                  System.out.println("translator> " + var3);
               }
            }
         } catch (IOException var14) {
            var14.printStackTrace();
         } finally {
            try {
               this.mLogFile.close();
            } catch (IOException var13) {
               var13.printStackTrace();
            }

         }

      }
   }
}
